@@ -14,36 +14,50 @@
 package org.apache.sedona.common;
 
 import com.google.common.geometry.S2CellId;
+import com.uber.h3core.exceptions.H3Exception;
+
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.sedona.common.geometryObjects.Circle;
 import org.apache.sedona.common.subDivide.GeometrySubDivider;
 import org.apache.sedona.common.utils.GeomUtils;
 import org.apache.sedona.common.utils.GeometryGeoHashEncoder;
 import org.apache.sedona.common.utils.GeometrySplitter;
+import org.apache.sedona.common.utils.H3Utils;
 import org.apache.sedona.common.utils.S2Utils;
-import org.geotools.geometry.jts.JTS;
-import org.geotools.referencing.CRS;
 import org.locationtech.jts.algorithm.MinimumBoundingCircle;
 import org.locationtech.jts.algorithm.hull.ConcaveHull;
-import org.locationtech.jts.geom.*;
+import org.locationtech.jts.geom.Coordinate;
+import org.locationtech.jts.geom.Geometry;
+import org.locationtech.jts.geom.GeometryCollection;
+import org.locationtech.jts.geom.GeometryFactory;
+import org.locationtech.jts.geom.LineString;
+import org.locationtech.jts.geom.LinearRing;
+import org.locationtech.jts.geom.MultiLineString;
+import org.locationtech.jts.geom.MultiPoint;
+import org.locationtech.jts.geom.MultiPolygon;
+import org.locationtech.jts.geom.Point;
+import org.locationtech.jts.geom.Polygon;
+import org.locationtech.jts.geom.PrecisionModel;
 import org.locationtech.jts.geom.impl.CoordinateArraySequence;
 import org.locationtech.jts.geom.util.GeometryFixer;
 import org.locationtech.jts.io.gml2.GMLWriter;
 import org.locationtech.jts.io.kml.KMLWriter;
 import org.locationtech.jts.linearref.LengthIndexedLine;
+import org.locationtech.jts.operation.distance.DistanceOp;
 import org.locationtech.jts.operation.distance3d.Distance3DOp;
 import org.locationtech.jts.operation.linemerge.LineMerger;
 import org.locationtech.jts.operation.valid.IsSimpleOp;
 import org.locationtech.jts.operation.valid.IsValidOp;
 import org.locationtech.jts.precision.GeometryPrecisionReducer;
 import org.locationtech.jts.simplify.TopologyPreservingSimplifier;
-import org.opengis.referencing.FactoryException;
-import org.opengis.referencing.NoSuchAuthorityCodeException;
-import org.opengis.referencing.crs.CoordinateReferenceSystem;
-import org.opengis.referencing.operation.MathTransform;
-import org.opengis.referencing.operation.TransformException;
 import org.wololo.jts2geojson.GeoJSONWriter;
-
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashSet;
+import java.util.List;
+import java.util.Set;
+import java.util.LinkedHashSet;
+import java.util.Collection;
 import java.util.stream.Collectors;
 
 import static com.google.common.geometry.S2.DBL_EPSILON;
@@ -181,36 +195,6 @@ public class Functions {
         return min == Double.MAX_VALUE ? null : min;
     }
 
-    public static Geometry transform(Geometry geometry, String sourceCRS, String targetCRS)
-        throws FactoryException, TransformException {
-        return transform(geometry, sourceCRS, targetCRS, false);
-    }
-
-    public static Geometry transform(Geometry geometry, String sourceCRS, String targetCRS, boolean lenient)
-        throws FactoryException, TransformException {
-
-        CoordinateReferenceSystem sourceCRScode = parseCRSString(sourceCRS);
-        CoordinateReferenceSystem targetCRScode = parseCRSString(targetCRS);
-        MathTransform transform = CRS.findMathTransform(sourceCRScode, targetCRScode, lenient);
-        return JTS.transform(geometry, transform);
-    }
-
-    private static CoordinateReferenceSystem parseCRSString(String CRSString)
-            throws FactoryException
-    {
-        try {
-            return CRS.decode(CRSString);
-        }
-        catch (NoSuchAuthorityCodeException e) {
-            try {
-                return CRS.parseWKT(CRSString);
-            }
-            catch (FactoryException ex) {
-                throw new FactoryException("First failed to read as a well-known CRS code: \n" + e.getMessage() + "\nThen failed to read as a WKT CRS string: \n" + ex.getMessage());
-            }
-        }
-    }
-
     public static Geometry flipCoordinates(Geometry geometry) {
         GeomUtils.flipCoordinates(geometry);
         return geometry;
@@ -447,6 +431,17 @@ public class Functions {
         return GEOMETRY_FACTORY.createLineString(coordinates.toArray(new Coordinate[0]));
     }
 
+    public static Geometry closestPoint(Geometry left, Geometry right) {
+        DistanceOp distanceOp = new DistanceOp(left, right);
+        try {
+            Coordinate[] closestPoints = distanceOp.nearestPoints();
+            return GEOMETRY_FACTORY.createPoint(closestPoints[0]);
+        }
+        catch (Exception e) {
+            throw new IllegalArgumentException("ST_ClosestPoint doesn't support empty geometry object.");
+        }
+    }
+
     public static Geometry concaveHull(Geometry geometry, double pctConvex, boolean allowHoles){
         ConcaveHull concave_hull = new ConcaveHull(geometry);
         concave_hull.setMaximumEdgeLengthRatio(pctConvex);
@@ -524,6 +519,14 @@ public class Functions {
         return circle;
     }
 
+    public static Pair<Geometry, Double> minimumBoundingRadius(Geometry geometry) {
+        MinimumBoundingCircle minimumBoundingCircle = new MinimumBoundingCircle(geometry);
+        Coordinate coods = minimumBoundingCircle.getCentre();
+        double radius = minimumBoundingCircle.getRadius();
+        Point centre = GEOMETRY_FACTORY.createPoint(coods);
+        return Pair.of(centre, radius);
+    }
+
     public static Geometry lineSubString(Geometry geom, double fromFraction, double toFraction) {
         double length = geom.getLength();
         LengthIndexedLine indexedLine = new LengthIndexedLine(geom);
@@ -554,6 +557,14 @@ public class Functions {
         return new GeometrySplitter(GEOMETRY_FACTORY).split(input, blade);
     }
 
+    public static Integer dimension(Geometry geometry) {
+        Integer dimension = geometry.getDimension();
+        // unknown dimension such as an empty GEOMETRYCOLLECTION
+        if (dimension < 0) {
+            dimension = 0;
+        }
+        return dimension;
+    }
 
     /**
      * get the coordinates of a geometry and transform to Google s2 cell id
@@ -575,6 +586,99 @@ public class Functions {
         return S2Utils.roundCellsToSameLevel(new ArrayList<>(cellIds), level).stream().map(S2CellId::id).collect(Collectors.toList()).toArray(new Long[cellIds.size()]);
     }
 
+    /**
+     * cover the geometry with H3 cells
+     * @param input the input geometry
+     * @param level the resolution of h3 cells
+     * @param fullCover whether enforce full cover or not.
+     * @return
+     */
+    public static Long[] h3CellIDs(Geometry input, int level, boolean fullCover) {
+        if (level < 0 || level > 15) {
+            throw new IllegalArgumentException("level must be between 0 and 15");
+        }
+        HashSet<Long> cellIds = new HashSet<>();
+        // first flat all GeometryCollection implementations into single Geometry types (Polygon, LineString, Point)
+        List<Geometry> geoms = GeomUtils.extractGeometryCollection(input);
+        for (Geometry geom : geoms) {
+            if (geom instanceof Polygon) {
+                cellIds.addAll(H3Utils.polygonToCells((Polygon) geom, level, fullCover));
+            } else if (geom instanceof LineString) {
+                cellIds.addAll(H3Utils.lineStringToCells((LineString) geom, level, fullCover));
+            } else if (geom instanceof Point){
+                cellIds.add(H3Utils.coordinateToCell(geom.getCoordinate(), level));
+            } else {
+                // if not type of polygon, point or lienSting, we cover its MBR
+                cellIds.addAll(H3Utils.polygonToCells((Polygon)geom.getEnvelope(), level, fullCover));
+            }
+        }
+        return cellIds.toArray(new Long[0]);
+    }
+
+    /**
+     * return the distance between 2 cells, if the native h3 function doesn't work, use our approximation function for shortest path and use the size - 1 as distance
+     * @param cell1 source cell
+     * @param cell2 destination cell
+     * @return
+     */
+    public static long h3CellDistance(long cell1, long cell2) {
+        int resolution = H3Utils.h3.getResolution(cell1);
+        if (resolution != H3Utils.h3.getResolution(cell2)) {
+            throw new IllegalArgumentException("The argument cells should be of the same resolution");
+        }
+        try {
+            return H3Utils.h3.gridDistance(cell1, cell2);
+        } catch (H3Exception e) {
+            // approximate if the original function hit error
+            return H3Utils.approxPathCells(
+                    H3Utils.cellToCoordinate(cell1),
+                    H3Utils.cellToCoordinate(cell2),
+                    resolution,
+                    true
+            ).size() - 1;
+        }
+    }
+
+    /**
+     * get the neighbor cells of the input cell by h3.gridDisk function
+     * @param cell: original cell
+     * @param k: the k number of rings spread from the original cell
+     * @param exactDistance: if exactDistance is true, it will only return the cells on the exact kth ring, else will return all 0 - kth neighbors
+     * @return
+     */
+    public static Long[] h3KRing(long cell, int k, boolean exactDistance) {
+        Set<Long> cells = new LinkedHashSet<>(H3Utils.h3.gridDisk(cell, k));
+        if (exactDistance && k > 0) {
+            List<Long> tbdCells = H3Utils.h3.gridDisk(cell, k - 1);
+            tbdCells.forEach(cells::remove);
+        }
+        return cells.toArray(new Long[0]);
+    }
+
+    /**
+     * get the neighbor cells of the input cell by h3.gridDisk function
+     * @param cells: the set of cells
+     * @return Multiple Polygons reversed
+     */
+    public static Geometry h3ToGeom(long[] cells) {
+        GeometryFactory geomFactory = new GeometryFactory();
+        Collection<Long> h3 = Arrays.stream(cells).boxed().collect(Collectors.toList());
+        return geomFactory.createMultiPolygon(
+                H3Utils.h3.cellsToMultiPolygon(h3, true).stream().map(
+                        shellHoles -> {
+                            List<LinearRing> rings = shellHoles.stream().map(
+                                    shell -> geomFactory.createLinearRing(shell.stream().map(latLng -> new Coordinate(latLng.lng, latLng.lat)).toArray(Coordinate[]::new))
+                            ).collect(Collectors.toList());
+                            LinearRing shell = rings.remove(0);
+                            if (rings.isEmpty()) {
+                                return geomFactory.createPolygon(shell);
+                            } else {
+                                return geomFactory.createPolygon(shell, rings.toArray(new LinearRing[0]));
+                            }
+                        }
+                ).toArray(Polygon[]::new)
+        );
+    }
 
     // create static function named simplifyPreserveTopology
     public static Geometry simplifyPreserveTopology(Geometry geometry, double distanceTolerance) {
@@ -585,6 +689,14 @@ public class Functions {
         return "ST_" + geometry.getGeometryType();
     }
 
+    public static String geometryTypeWithMeasured(Geometry geometry) {
+        String geometryType = geometry.getGeometryType().toUpperCase();
+        if (GeomUtils.isMeasuredGeometry(geometry)) {
+            geometryType += "M";
+        }
+        return geometryType;
+    }
+
     public static Geometry startPoint(Geometry geometry) {
         if (geometry instanceof LineString) {
             LineString line = (LineString) geometry;
@@ -646,6 +758,28 @@ public class Functions {
         return GeometrySubDivider.subDivide(geometry, maxVertices);
     }
 
+    public static Geometry makeLine(Geometry geom1, Geometry geom2) {
+        Geometry[] geoms = new Geometry[]{geom1, geom2};
+        return makeLine(geoms);
+    }
+
+    public static Geometry makeLine(Geometry[] geoms) {
+        ArrayList<Coordinate> coordinates = new ArrayList<>();
+        for (Geometry geom : geoms) {
+            if (geom instanceof Point || geom instanceof MultiPoint || geom instanceof LineString) {
+                for (Coordinate coord : geom.getCoordinates()) {
+                    coordinates.add(coord);
+                }
+            }
+            else {
+                throw new IllegalArgumentException("ST_MakeLine only supports Point, MultiPoint and LineString geometries");
+            }
+        }
+
+        Coordinate[] coords = coordinates.toArray(new Coordinate[0]);
+        return GEOMETRY_FACTORY.createLineString(coords);
+    }
+
     public static Geometry makePolygon(Geometry shell, Geometry[] holes) {
         try {
             if (holes != null) {
@@ -673,6 +807,14 @@ public class Functions {
         }
     }
 
+    public static Geometry makepolygonWithSRID(Geometry lineString, Integer srid) {
+        Geometry geom = makePolygon(lineString, null);
+        if(geom != null) {
+            geom.setSRID(srid);
+        }
+        return geom;
+    }
+
     public static Geometry createMultiGeometry(Geometry[] geometries) {
         if (geometries.length > 1){
             return GEOMETRY_FACTORY.buildGeometry(Arrays.asList(geometries));
@@ -859,7 +1001,7 @@ public class Functions {
     }
 
     public static Geometry force3D(Geometry geometry) {
-       return GeomUtils.get3DGeom(geometry, 0.0);
+        return GeomUtils.get3DGeom(geometry, 0.0);
     }
 
     public static Integer nRings(Geometry geometry) throws Exception {
@@ -896,6 +1038,21 @@ public class Functions {
         return geometry;
     }
 
+    public static Geometry affine(Geometry geometry, double a, double b, double c, double d, double e, double f, double g, double h, double i, double xOff, double yOff,
+            double zOff) {
+        if (!geometry.isEmpty()) {
+            GeomUtils.affineGeom(geometry, a, b, c, d, e, f, g, h, i, xOff, yOff, zOff);
+        }
+        return geometry;
+    }
+
+    public static Geometry affine(Geometry geometry, double a, double b, double d, double e, double xOff, double yOff) {
+        if (!geometry.isEmpty()) {
+            GeomUtils.affineGeom(geometry, a, b, null, d, e, null, null, null, null, xOff, yOff, null);
+        }
+        return geometry;
+    }
+
     public static Geometry geometricMedian(Geometry geometry, double tolerance, int maxIter, boolean failIfNotConverged) throws Exception {
         String geometryType = geometry.getGeometryType();
         if(!(Geometry.TYPENAME_POINT.equals(geometryType) || Geometry.TYPENAME_MULTIPOINT.equals(geometryType))) {
@@ -931,4 +1088,83 @@ public class Functions {
         return geometricMedian(geometry, DEFAULT_TOLERANCE, DEFAULT_MAX_ITER, false);
     }
 
+    public static double frechetDistance(Geometry g1, Geometry g2) {
+        return GeomUtils.getFrechetDistance(g1, g2);
+    }
+
+    public static boolean isCollection(Geometry geometry) {
+        String geoType = geometry.getGeometryType();
+        return Geometry.TYPENAME_GEOMETRYCOLLECTION.equalsIgnoreCase(geoType) ||
+                Geometry.TYPENAME_MULTIPOINT.equalsIgnoreCase(geoType) ||
+                Geometry.TYPENAME_MULTIPOLYGON.equalsIgnoreCase(geoType) ||
+                Geometry.TYPENAME_MULTILINESTRING.equalsIgnoreCase(geoType);
+    }
+
+    public static Geometry boundingDiagonal(Geometry geometry) {
+        if (geometry.isEmpty()) {
+            return GEOMETRY_FACTORY.createLineString();
+        }else {
+            Double startX = null, startY = null, startZ = null,
+                    endX = null, endY = null, endZ = null;
+            boolean is3d = !Double.isNaN(geometry.getCoordinate().z);
+            for (Coordinate currCoordinate : geometry.getCoordinates()) {
+                startX = startX == null ? currCoordinate.getX() : Math.min(startX, currCoordinate.getX());
+                startY = startY == null ? currCoordinate.getY() : Math.min(startY, currCoordinate.getY());
+
+                endX = endX == null ? currCoordinate.getX() : Math.max(endX, currCoordinate.getX());
+                endY = endY == null ? currCoordinate.getY() : Math.max(endY, currCoordinate.getY());
+                if (is3d) {
+                    Double geomZ = currCoordinate.getZ();
+                    startZ = startZ == null ? currCoordinate.getZ() : Math.min(startZ, currCoordinate.getZ());
+                    endZ = endZ == null ? currCoordinate.getZ() : Math.max(endZ, currCoordinate.getZ());
+                }
+            }
+            Coordinate startCoordinate;
+            Coordinate endCoordinate;
+            if (is3d) {
+                startCoordinate = new Coordinate(startX, startY, startZ);
+                endCoordinate = new Coordinate(endX, endY, endZ);
+            }else {
+                startCoordinate = new Coordinate(startX, startY);
+                endCoordinate = new Coordinate(endX, endY);
+            }
+            return GEOMETRY_FACTORY.createLineString(new Coordinate[] {startCoordinate, endCoordinate});
+        }
+    }
+
+    public static double angle(Geometry point1, Geometry point2, Geometry point3, Geometry point4) throws IllegalArgumentException {
+        if (point3 == null && point4 == null) return Functions.angle(point1, point2);
+        else if (point4 == null) return Functions.angle(point1, point2, point3);
+        if (GeomUtils.isAnyGeomEmpty(point1, point2, point3, point4)) throw new IllegalArgumentException("ST_Angle cannot support empty geometries.");
+        if (!(point1 instanceof Point && point2 instanceof Point && point3 instanceof Point && point4 instanceof Point)) throw new IllegalArgumentException("ST_Angle supports either only POINT or only LINESTRING geometries.");
+        return GeomUtils.calcAngle(point1.getCoordinate(), point2.getCoordinate(), point3.getCoordinate(), point4.getCoordinate());
+    }
+
+    public static double angle(Geometry point1, Geometry point2, Geometry point3) throws IllegalArgumentException {
+        if (GeomUtils.isAnyGeomEmpty(point1, point2, point3)) throw new IllegalArgumentException("ST_Angle cannot support empty geometries.");
+        if (!(point1 instanceof Point && point2 instanceof Point && point3 instanceof Point)) throw new IllegalArgumentException("ST_Angle supports either only POINT or only LINESTRING geometries.");
+        return GeomUtils.calcAngle(point2.getCoordinate(), point1.getCoordinate(), point2.getCoordinate(), point3.getCoordinate());
+    }
+
+    public static double angle(Geometry line1, Geometry line2) throws IllegalArgumentException {
+        if (GeomUtils.isAnyGeomEmpty(line1, line2)) throw new IllegalArgumentException("ST_Angle cannot support empty geometries.");
+        if (!(line1 instanceof LineString && line2 instanceof LineString)) throw new IllegalArgumentException("ST_Angle supports either only POINT or only LINESTRING geometries.");
+        Coordinate[] startEndLine1 = GeomUtils.getStartEndCoordinates(line1);
+        Coordinate[] startEndLine2 = GeomUtils.getStartEndCoordinates(line2);
+        assert startEndLine1 != null;
+        assert startEndLine2 != null;
+        return GeomUtils.calcAngle(startEndLine1[0], startEndLine1[1], startEndLine2[0], startEndLine2[1]);
+    }
+
+    public static double degrees(double angleInRadian) {
+        return GeomUtils.toDegrees(angleInRadian);
+    }
+
+    public static Double hausdorffDistance(Geometry g1, Geometry g2, double densityFrac) throws Exception {
+        return GeomUtils.getHausdorffDistance(g1, g2, densityFrac);
+    }
+
+    public static Double hausdorffDistance(Geometry g1, Geometry g2) throws Exception{
+        return GeomUtils.getHausdorffDistance(g1, g2, -1);
+    }
 }
